@@ -1,4 +1,4 @@
-import greenlet, logging, os, os.path, msgpack
+import greenlet, logging, os, os.path
 from collections import deque
 from mixins import mixins
 from util import VimError, VimExit
@@ -28,11 +28,13 @@ class Client(object):
     """
     Neovim client. It depends on a rpc stream, an object that implements four
     methods:
+        - configure_types(vim, types): Configure the stream's
+            serializer/deserializer with custom type information
         - loop_start(request_cb, notification_cb error_cb): Start the event
             loop to receive rpc requests and notifications
         - loop_stop(): Stop the event loop
-        - send(method, args, response_cb): Send a method call with args,
-            and invoke response_cb when the response is available
+        - send(method, args, response_cb): Send a method call with args, and
+            invoke response_cb when the response is available
         - post(name, args): Post a notification from another thread
     """
     def __init__(self, stream, vim_compatible=False):
@@ -96,12 +98,6 @@ class Client(object):
         if err:
             raise VimError(err)
 
-        if expected_type and hasattr(self.vim, expected_type):
-            # result should be a handle, wrap in it's specialized class
-            klass = getattr(self.vim, expected_type)
-            result = klass(self.vim, result)
-            klass.initialize(result)
-
         return result
 
 
@@ -276,50 +272,52 @@ class Client(object):
         if self.vim:
             # Only need to do this once
             return
-        channel_id, api = self.rpc_request(0, [])
-        api = msgpack.unpackb(api)
+        channel_id, api = self.rpc_request("vim_get_api_info", [])
         # The 'Vim' class is the main entry point of the api
-        classes = {'vim': type('Vim', (), {})}
-        setattr(classes['vim'], 'loop_start',
+        types = {'vim': type('Vim', (), {})}
+        setattr(types['vim'], 'loop_start',
                 lambda s, *args, **kwargs: self.loop_start(*args, **kwargs))
-        setattr(classes['vim'], 'loop_stop',
+        setattr(types['vim'], 'loop_stop',
                 lambda s, *args, **kwargs: self.loop_stop(*args, **kwargs))
-        setattr(classes['vim'], 'next_message',
+        setattr(types['vim'], 'next_message',
                 lambda s, *args, **kwargs: self.next_message(*args, **kwargs))
-        setattr(classes['vim'], 'post',
+        setattr(types['vim'], 'post',
                 lambda s, *args, **kwargs: self.post(*args, **kwargs))
-        # Build classes for manipulating the remote structures, assigning to a
+        # Build types for manipulating the remote structures, assigning to a
         # dict using lower case names as keys, so we can easily match methods
         # in the API.
-        for cls in api['classes']:
+        for cls in api['types']:
             klass = type(cls + 'Base', (Remote,), {})
             # Methods of this class will pass an integer representing the
             # remote object as first argument
-            classes[cls.lower()] = klass
+            types[cls.lower()] = klass
         # now build function wrappers
         for function in api['functions']:
             # Split the name on underscores, the first part is the class name,
             # the remaining is the function name
             class_name, method_name = function['name'].split('_', 1)
             generate_wrapper(self,
-                             classes[class_name],
+                             types[class_name],
                              method_name,
-                             function['id'],
+                             function['name'],
                              function['return_type'],
                              function['parameters'])
         if self.vim_compatible:
-            make_vim_compatible(classes['vim'])
-        # Now apply all available mixins to the generated classes
+            make_vim_compatible(types['vim'])
+        # Now apply all available mixins to the generated types
         for name, mixin in mixins.items():
-            classes[name] = type(mixin.__name__, (classes[name], mixin,), {})
+            types[name] = type(mixin.__name__, (types[name], mixin,), {})
         # Create the 'vim object', which is a singleton of the 'Vim' class
-        self.vim = classes['vim']()
+        self.vim = types['vim']()
         # Initialize with some useful attributes
-        classes['vim'].initialize(self.vim, classes, channel_id)
+        types['vim'].initialize(self.vim, types, channel_id, api)
         # Add attributes for each other class
-        for name, klass in classes.items():
+        for name, klass in types.items():
             if name != 'vim':
                 setattr(self.vim, klass.__name__, klass)
+        # Configure the rpc stream with type information
+        self.stream.configure(self.vim)
+        self.vim
 
 
 def generate_wrapper(client, klass, name, fid, return_type, parameters):
@@ -341,10 +339,6 @@ def generate_wrapper(client, klass, name, fid, return_type, parameters):
         argv = []
         # fill with positional arguments
         for i, arg in enumerate(args):
-            if hasattr(client.vim, parameters[i][0]):
-                # If the type is a remote object class, we use it's remote
-                # handle instead
-                arg = arg._handle
             # Add to the argument vector 
             argv.append(arg)
         return client.rpc_request(fid, argv, return_type)
